@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -7,12 +7,14 @@
 # ----------------------------------------------------------------------------
 
 from ._normalize import rarefy
-from ._subsample import subsample
+from ._subsample_ids import subsample_ids
 from ._transform import (presence_absence, relative_frequency, transpose)
-from ._summarize import (summarize, tabulate_seqs)
+from ._summarize import (summarize, tabulate_seqs, tabulate_sample_frequencies,
+                         tabulate_feature_frequencies, summarize_plus)
 from ._merge import (merge, merge_seqs, merge_taxa, overlap_methods)
 from ._filter import (filter_samples, filter_features, filter_seqs,
                       filter_features_conditionally)
+from ._split import split
 from ._core_features import core_features
 from ._group import group
 from ._rename import rename_ids
@@ -26,5 +28,7 @@ __all__ = ['rarefy', 'presence_absence', 'relative_frequency', 'transpose',
            'summarize', 'merge', 'merge_seqs', 'filter_samples',
            'filter_features', 'merge_taxa', 'tabulate_seqs', 'overlap_methods',
            'core_features', 'group', 'heatmap', 'heatmap_choices',
-           'filter_seqs', 'subsample', 'rename_ids',
-           'filter_features_conditionally']
+           'filter_seqs', 'subsample_ids', 'rename_ids',
+           'filter_features_conditionally', 'split',
+           'tabulate_feature_frequencies', 'tabulate_sample_frequencies',
+           'summarize_plus']
