@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -8,16 +8,17 @@
 
 from qiime2.plugin import (Plugin, Int, Float, Range, Metadata, Str, Bool,
                            Choices, MetadataColumn, Categorical, List,
-                           Citations, TypeMatch, TypeMap)
+                           Citations, TypeMatch, TypeMap, Collection,
+                           Visualization)
 
 from q2_types.feature_table import (
     FeatureTable, Frequency, RelativeFrequency, PresenceAbsence, Composition)
 from q2_types.feature_data import (
     FeatureData, Sequence, Taxonomy, AlignedSequence)
+from q2_types.metadata import ImmutableMetadata
 
 import q2_feature_table
 import q2_feature_table._examples as ex
-
 citations = Citations.load('citations.bib', package='q2_feature_table')
 plugin = Plugin(
     name='feature-table',
@@ -29,6 +30,7 @@ plugin = Plugin(
                  'by feature tables, such as filtering, merging, and '
                  'transforming tables.')
 )
+T1 = TypeMatch([Frequency, RelativeFrequency, PresenceAbsence, Composition])
 
 plugin.methods.register_function(
     function=q2_feature_table.rarefy,
@@ -56,7 +58,7 @@ plugin.methods.register_function(
 )
 
 plugin.methods.register_function(
-    function=q2_feature_table.subsample,
+    function=q2_feature_table.subsample_ids,
     inputs={'table': FeatureTable[Frequency]},
     parameters={'subsampling_depth': Int % Range(1, None),
                 'axis': Str % Choices(['sample', 'feature'])},
@@ -121,10 +123,10 @@ plugin.methods.register_function(
 
 plugin.methods.register_function(
     function=q2_feature_table.transpose,
-    inputs={'table': FeatureTable[Frequency]},
+    inputs={'table': FeatureTable[T1]},
     parameters={},
     outputs=[('transposed_feature_table',
-             FeatureTable[Frequency])],
+             FeatureTable[T1])],
     input_descriptions={
         'table': 'The feature table to be transposed.'
     },
@@ -174,15 +176,19 @@ plugin.methods.register_function(
     examples={'group_samples': ex.feature_table_group_samples}
 )
 
+# maps input types to relevant overlap methods and output types
 i_table, p_overlap_method, o_table = TypeMap({
     (FeatureTable[Frequency],
-     Str % Choices(sorted(q2_feature_table.overlap_methods()))):
+     Str % Choices(sorted(q2_feature_table.overlap_methods() - {'union'}))):
     FeatureTable[Frequency],
     (FeatureTable[RelativeFrequency],
-     # We don't want to allow summing of RelativeFrequency tables, so remove
-     # that option from the overlap methods
-     Str % Choices(sorted(q2_feature_table.overlap_methods() - {'sum'}))):
-    FeatureTable[RelativeFrequency]
+     Str % Choices(sorted(q2_feature_table.overlap_methods()
+                          - {'union', 'sum'}))):
+    FeatureTable[RelativeFrequency],
+    (FeatureTable[PresenceAbsence],
+     Str % Choices(sorted(q2_feature_table.overlap_methods()
+                          - {'average', 'sum'}))):
+    FeatureTable[PresenceAbsence]
 })
 
 plugin.methods.register_function(
@@ -260,7 +266,6 @@ plugin.methods.register_function(
     }
 )
 
-T1 = TypeMatch([Frequency, RelativeFrequency, PresenceAbsence, Composition])
 
 plugin.methods.register_function(
     function=q2_feature_table.rename_ids,
@@ -308,7 +313,8 @@ plugin.methods.register_function(
                 'metadata': Metadata,
                 'where': Str,
                 'exclude_ids': Bool,
-                'filter_empty_features': Bool},
+                'filter_empty_features': Bool,
+                'allow_empty_table': Bool},
     outputs=[('filtered_table', FeatureTable[T1])],
     input_descriptions={
         'table': 'The feature table from which samples should be filtered.'
@@ -338,6 +344,9 @@ plugin.methods.register_function(
                        'table instead of being retained.',
         'filter_empty_features': 'If true, features which are not present in '
                                  'any retained samples are dropped.',
+        'allow_empty_table': 'If true, the filtered table may be empty. '
+                             'Default behavior is to raise an error if the '
+                             'filtered table is empty.'
     },
     output_descriptions={
         'filtered_table': 'The resulting feature table filtered by sample.'
@@ -363,8 +372,8 @@ plugin.methods.register_function(
     function=q2_feature_table.filter_features_conditionally,
     inputs={'table': FeatureTable[T1]},
     parameters={'prevalence': Float % Range(0, 1),
-                'abundance': Float % Range(0, 1)
-                },
+                'abundance': Float % Range(0, 1),
+                'allow_empty_table': Bool},
     outputs=[('filtered_table', FeatureTable[T1])],
     input_descriptions={
         'table': 'The feature table from which features should be filtered.'
@@ -374,7 +383,10 @@ plugin.methods.register_function(
                       'retained.'),
         'prevalence': ('The minimum portion of samples that a feature '
                        'must have a relative abundance of at least '
-                       '`abundance` to be retained.')
+                       '`abundance` to be retained.'),
+        'allow_empty_table': 'If true, the filtered table may be empty. '
+                             'Default behavior is to raise an error if the '
+                             'filtered table is empty.'
     },
     output_descriptions={
         'filtered_table': 'The resulting feature table filtered by feature.'
@@ -403,7 +415,8 @@ plugin.methods.register_function(
                 'metadata': Metadata,
                 'where': Str,
                 'exclude_ids': Bool,
-                'filter_empty_samples': Bool},
+                'filter_empty_samples': Bool,
+                'allow_empty_table': Bool},
     outputs=[('filtered_table', FeatureTable[Frequency])],
     input_descriptions={
         'table': 'The feature table from which features should be filtered.'
@@ -433,6 +446,9 @@ plugin.methods.register_function(
                        'table instead of being retained.',
         'filter_empty_samples': 'If true, drop any samples where none of the '
                                 'retained features are present.',
+        'allow_empty_table': 'If true, the filtered table may be empty. '
+                                'Default behavior is to raise an error if the '
+                                'filtered table is empty.'
     },
     output_descriptions={
         'filtered_table': 'The resulting feature table filtered by feature.'
@@ -495,8 +511,7 @@ plugin.methods.register_function(
 
 plugin.visualizers.register_function(
     function=q2_feature_table.summarize,
-    inputs={'table': FeatureTable[Frequency | RelativeFrequency |
-                                  PresenceAbsence]},
+    inputs={'table': FeatureTable[Frequency | PresenceAbsence]},
     parameters={'sample_metadata': Metadata},
     input_descriptions={'table': 'The feature table to be summarized.'},
     parameter_descriptions={'sample_metadata': 'The sample metadata.'},
@@ -509,10 +524,19 @@ plugin.visualizers.register_function(
 
 plugin.visualizers.register_function(
     function=q2_feature_table.tabulate_seqs,
-    inputs={'data': FeatureData[Sequence | AlignedSequence]},
-    parameters={},
-    input_descriptions={'data': 'The feature sequences to be tabulated.'},
-    parameter_descriptions={},
+    inputs={'data': FeatureData[Sequence | AlignedSequence],
+            'taxonomy': Collection[FeatureData[Taxonomy]]},
+    parameters={'metadata': Metadata,
+                'merge_method': Str % Choices(
+                    ['strict', 'union', 'intersect'])},
+    input_descriptions={
+        'data': 'The feature sequences to be tabulated.',
+        'taxonomy': 'The taxonomic classifications of the tabulated features.'
+        },
+    parameter_descriptions={
+     'metadata': 'Any additional metadata for the tabulated features.',
+     'merge_method': 'Method that joins data sets'
+    },
     name='View sequence associated with each feature',
     description="Generate tabular view of feature identifier to sequence "
                 "mapping, including links to BLAST each sequence against "
@@ -520,6 +544,10 @@ plugin.visualizers.register_function(
     citations=[citations['NCBI'], citations['NCBI-BLAST']],
     examples={
         'feature_table_tabulate_seqs': ex.feature_table_tabulate_seqs,
+        'feature_table_tabulate_seqs_single_taxon':
+            ex.feature_table_tabulate_seqs_single_taxon,
+        'feature_table_tabulate_seqs_multi_taxon':
+            ex.feature_table_tabulate_seqs_multi_taxon
     }
 )
 
@@ -607,3 +635,91 @@ plugin.visualizers.register_function(
     },
     citations=[citations['Hunter2007Matplotlib']]
 )
+
+plugin.methods.register_function(
+    function=q2_feature_table.split,
+    inputs={'table': FeatureTable[T1]},
+    parameters={
+        'metadata': MetadataColumn[Categorical],
+        'filter_empty_features': Bool
+    },
+    outputs=[
+        ('tables', Collection[FeatureTable[T1]])
+    ],
+    input_descriptions={
+        'table': 'The table to split.'
+    },
+    parameter_descriptions={
+        'metadata': 'A column defining the groups. Each unique value will '
+                    'define a split feature table.',
+        'filter_empty_features': 'If true, features which are not present in '
+                                 'a split feature table are dropped.',
+    },
+    output_descriptions={
+        'tables': ('Directory where feature tables split based on metadata '
+                   'values should be written.')
+    },
+    name='Split one feature table into many',
+    description='Splits one feature table into many feature tables, where '
+                'splits are defined by values in metadata column.',
+    examples={}
+)
+
+plugin.methods.register_function(
+    function=q2_feature_table.tabulate_feature_frequencies,
+    inputs={'table': FeatureTable[Frequency | PresenceAbsence |
+                                  RelativeFrequency]},
+    parameters={},
+    outputs={'feature_frequencies': ImmutableMetadata},
+    input_descriptions={
+        'table': 'The input feature table.'
+    },
+    output_descriptions={
+        'feature_frequencies': 'Per-sample and total frequencies per feature.'
+    },
+    name='Tabulate feature frequencies',
+    description='Tabulate sample count and total frequency per feature.',
+    examples={'feature_table_tabulate_feature_frequencies':
+              ex.feature_table_tabulate_feature_freqs}
+)
+
+plugin.methods.register_function(
+    function=q2_feature_table.tabulate_sample_frequencies,
+    inputs={'table': FeatureTable[Frequency | PresenceAbsence |
+                                  RelativeFrequency]},
+    parameters={},
+    outputs={'sample_frequencies': ImmutableMetadata},
+    input_descriptions={
+        'table': 'The input feature table.'
+    },
+    output_descriptions={
+        'sample_frequencies': 'Observed feature count and total' +
+        ' frequencies per sample.'
+    },
+    name='Tabulate sample frequencies',
+    description='Tabulate feature count and total frequency per sample.',
+    examples={'feature_table_tabulate_sample_frequencies':
+              ex.feature_table_tabulate_sample_freqs}
+)
+
+plugin.pipelines.register_function(
+    function=q2_feature_table.summarize_plus,
+    inputs={'table': FeatureTable[Frequency | PresenceAbsence]},
+    parameters={'metadata': Metadata},
+    outputs={'feature_frequencies': ImmutableMetadata,
+             'sample_frequencies': ImmutableMetadata,
+             'summary': Visualization},
+    input_descriptions={
+        'table': 'The feature table to be summarized.'
+    },
+    parameter_descriptions={'metadata': 'The sample metadata.'},
+    output_descriptions={'feature_frequencies': 'Per-sample and total ' +
+                         'frequencies per feature.',
+                         'sample_frequencies': 'Observed feature count and ' +
+                         'total frequencies per sample.',
+                         'summary': 'Visual summary of feature table'},
+    name="Summarize table plus",
+    description="Generate visual and tabular summaries of a feature table. "
+                "Tabulate sample and feature frequencies.",
+    examples={'feature_table_summarize_plus': ex.feature_table_summarize_plus}
+)
