@@ -9,6 +9,8 @@
 import os
 from unittest import TestCase, main
 import tempfile
+import re
+import json
 
 import skbio
 import biom
@@ -383,6 +385,23 @@ class SummarizeTests(TestCase):
             index_fp = os.path.join(output_dir, 'index.html')
             self.assertTrue(os.path.exists(index_fp))
 
+            sample_frequency_fp = os.path.join(output_dir,
+                                               'sample-frequency-detail.html')
+            self.assertTrue(os.path.exists(sample_frequency_fp))
+            rx = (r'<script id="table-data" type="application/json">' +
+                  r'\n.*[^}]*.*\n</script>')
+
+            with open(sample_frequency_fp) as fi:
+                text = fi.read()
+                tbl_rx = re.compile(rx)
+                tbl = tbl_rx.findall(text)[0].split('\n')[1].strip()
+
+                sample_ids = json.loads(tbl)['Frequency'].keys()
+
+                self.assertTrue('S1' in sample_ids)
+                self.assertTrue('S2' in sample_ids)
+                self.assertTrue('S3' in sample_ids)
+
     def test_frequency_ranges_are_zero(self):
         table = biom.Table(np.array([[25, 25, 25], [25, 25, 25]]),
                            ['O1', 'O2'],
